/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "statisticssummary.h"
#include "statisticssummary.moc"

#include <esqldatabase.h>

#include "meganizer.h"
#include "mediaplugin.h"
#include <escaledimage.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QPainter>
#include <QVariant>
#include <QPicture>
#include <QGridLayout>

#include <cmath>

StatisticsSummary::StatisticsSummary ( QList<MediaPlugin*> plugins, QWidget* parent, Qt::WindowFlags flags )
	: EWidget ( parent, flags )
{
	setObjectName("statisticsSummaryWidget");
	setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
	setMinimumSize(200, 200);
	
	unsigned int myUserId = 0;
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.prepare("SELECT * FROM "+ Meganizer::group() +"_users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUserId = query.record().value("id").toUInt();
		
		// build mapping for our media types
		query.exec("SELECT COUNT(mats.id), types.name "
							 "  FROM viewmaterials mats, "+ Meganizer::group() +"_materialtypes types"
							 "  WHERE mats.materialtypesId=types.id"
							 "  GROUP BY materialtypesId"
							 "  ORDER BY COUNT(mats.id) DESC, types.name ASC");
		while(query.next()){
			QString type;
			foreach(MediaPlugin *plugin, plugins){
				QStringList result = plugin->translateMaterialType(query.record().value("name").toString());
				if(result.size() == 2 && !result[0].isEmpty()){
					type = result[0];
					break;
				}
			}
			if(!type.isEmpty()){
				ValuePair pair;
				pair.name = type;
				pair.value = query.record().value("COUNT(mats.id)").toInt();
				mapTypeCount << pair;
			}
		}
		
		// build user medias
		query.exec("SELECT COUNT(mats.id), users.name, users.givenName"
							 "  FROM viewmaterials mats, "+ Meganizer::group() +"_users users"
							 "  WHERE mats.usersId=users.id"
							 "  GROUP BY mats.usersId"
							 "  ORDER BY COUNT(mats.id) DESC, users.name ASC");
		while(query.next()){
			QString name = query.record().value("name").toString();
			QString givenName = query.record().value("givenName").toString();
			if(!givenName.isEmpty())
				name += ", "+ givenName.left(1);
			ValuePair pair;
			pair.name = name;
			pair.value = query.record().value("COUNT(mats.id)").toInt();
			mapUserCount << pair;
		}
		
		if(closeDB)
			db.close();
	}

	PI = acos(-1);
	margin = 100;
	
	colorNames << "darkRed" << "darkGreen" << "darkBlue";
	colorNames << "red" << "green" << "blue" << "yellow" << "black";
	colorNames << "darkCyan" << "darkMagenta" <<  "darkGrey";
	colorNames << "cyan" << "magenta";

	QGridLayout *ml = new QGridLayout(this);
	setLayout(ml);
	mediaTypeLabel = new EScaledImage(this);
	ml->addWidget(mediaTypeLabel, 0, 0);
	userLabel = new EScaledImage(this);
	ml->addWidget(userLabel);
	
	// draw type chart
	mediaTypeLabel->setImage(drawPie(mapTypeCount));

	// draw user chart
	userLabel->setImage(drawPie(mapUserCount));
}

QImage StatisticsSummary::drawPie(QList< ValuePair > quantities)
{
	QPixmap picture(QSize(2500, 1000));
	QPainter painter;
	painter.begin(&picture);
	
	QFont textFont = font();
	textFont.setPointSize(75);
	textFont.setBold(true);
	painter.setFont(textFont);
	
	// calculate the number of rows
	// set the line space
	float lineSpace = 0.3 * float(painter.fontMetrics().height()) + float(painter.fontMetrics().height());
	// Decrease the height of the picture with the margin at the top and the bottom.
	// Afterwards subtract the height needed for the Total line at the bottom with
	// the needed space above
	int rows = floor((float(picture.height()) - 2 * float(margin) - lineSpace) / lineSpace);
	if(rows > colorNames.size())
		rows = colorNames.size();
	quantities = restrictNumberOfEntries(quantities, rows);
	
	QRect chartRect = picture.rect();
	int square = chartRect.width() - 3 * margin;
	// make them a square
	if(square > chartRect.height() - 2 * margin)
		square = chartRect.height() - 2 * margin;
	// setting pie rect
	QRect pieRect;
	pieRect.setLeft(chartRect.left() + margin);
	pieRect.setTop(chartRect.top() + margin);
	pieRect.setWidth(square);
	pieRect.setHeight(square);

	double radius = float(pieRect.width()) / 2.0; 
	double area = PI*pow(radius, 2);
	QPoint origin;
	origin.setY(pieRect.y() + radius);
	origin.setX(pieRect.x() + radius);
	
	float total = 0;
	QString textLength;
	foreach(ValuePair pair, quantities){
		total += pair.value;
		QString check = pair.name +" - 100%";
		if(textLength.length() < check.length())
			textLength = check;
	}
	
	// calc angles
	QList<QList<int> > angles;
	int startAngle = 0;
	foreach(ValuePair pair, quantities){
		double piece = area * pair.value / total;
		double spanAngle = 16.0 * piece * 360.0 / PI / pow(radius, 2);
		QList<int> values;
		values << startAngle;
		values << spanAngle;
		angles.append(values);
		startAngle += spanAngle;
	}
	
	// drawing background white
// 	chartRect.setWidth(pieRect.width() + 2 * margin + painter.fontMetrics().width(textLength));
	painter.fillRect(chartRect, palette().background());
	
	// draw pie
	QColor brushColor;
	for(int i = 0; i < angles.size(); i++){
		if(i <= colorNames.size())
			brushColor.setNamedColor(colorNames[i]);
		painter.setBrush(QBrush(brushColor));
		painter.drawPie(pieRect, angles[i][0], angles[i][1]);
	}

	// draw legend
	QPoint textPoint = origin;
	textPoint.setX(pieRect.right() + 2 * margin);
	textPoint.setY(chartRect.top() + margin + painter.fontMetrics().height());
	int i = 0;
	foreach(ValuePair pair, quantities){
		if(i <= colorNames.size())
			brushColor.setNamedColor(colorNames[i]);

 		painter.setPen(brushColor);
		painter.drawText(textPoint, pair.name +" - "+ QString::number(float(pair.value) / total * 100.0, 'f', 0) + "%");
		textPoint.setY(textPoint.y() + lineSpace);
		i++;
	}

	painter.setPen(palette().foreground().color());
	painter.drawText(textPoint.x(), chartRect.bottom() - margin, tr("Total: %1").arg(total));
	painter.end();
	return picture.toImage();
}

QList< ValuePair > StatisticsSummary::restrictNumberOfEntries(QList< ValuePair > quantities, int rows)
{
	QList<ValuePair> result;
	
	// reduce rows by one, because the last row is the Others entry
	rows--;
	for(int i = 0; i < quantities.size(); i++){
		if(i < rows){
			// just append the first three result to result
			result << quantities[i];
		}
		else{
			if(i == rows){
				// add the others pair first
				result << ValuePair();
				result[rows].name = tr("Others");
			}
			result[rows].value += quantities[i].value;
		}
	}
	return result;
}
