/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef STATISTICSSUMMARY_H
#define STATISTICSSUMMARY_H

#include <ewidget.h>

class EScaledImage;
class MediaPlugin;

#include <QHash>

class ValuePair {
	public:
		ValuePair(){ value = 0; };
		QString name;
		float value;
};

/**
	* @brief Displays some statistics about the database.
	*/
class StatisticsSummary : public EWidget
{
	Q_OBJECT
	public:
		StatisticsSummary(QList<MediaPlugin*> plugins, QWidget *parent = 0, Qt::WindowFlags flags = 0);
		
	private:
		/**
			* @brief Draws a pie with a legend.
			* 
			* The values in quantities are concatenated to a value others, if
			* more values than possible rows are given. Also the number of defined colors
			* is used to correct the maximum number of rows.
			*/
		QImage drawPie(QList< ValuePair > quantities);
		/**
			* @brief All others than the first three are concatenated into a value others.
			* 
			* This is done, because with actuall setup, only three rows are possible.
			*/
		QList<ValuePair> restrictNumberOfEntries(QList<ValuePair> quantities, int rows);

		// we use QHash here, because we've ordered the items already
		QList<ValuePair> mapTypeCount;
		QList<ValuePair> mapUserCount;
		double PI;
		int margin;
		QStringList colorNames;
		EScaledImage *mediaTypeLabel, *userLabel;
};

#endif // STATISTICSSUMMARY_H
